import * as i0 from '@angular/core';
import { forwardRef, Directive, Input, HostBinding, NgModule, ApplicationRef, ComponentFactoryResolver, ElementRef, Component } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { FloatingMenuPlugin, FloatingMenuPluginKey } from '@tiptap/extension-floating-menu';
import { BubbleMenuPlugin, BubbleMenuPluginKey } from '@tiptap/extension-bubble-menu';
import { NodeView } from '@tiptap/core';

class EditorDirective {
    constructor(el, _renderer) {
        this.el = el;
        this._renderer = _renderer;
        this.outputFormat = 'html';
        this.onChange = () => { };
        this.onTouched = () => { };
        this.handleChange = ({ transaction }) => {
            if (!transaction.docChanged) {
                return;
            }
            if (this.outputFormat === 'html') {
                this.onChange(this.editor.getHTML());
                return;
            }
            this.onChange(this.editor.getJSON());
        };
    }
    // Writes a new value to the element.
    // This methods is called when programmatic changes from model to view are requested.
    writeValue(value) {
        if (!value) {
            return;
        }
        if (!this.outputFormat && typeof value === 'string') {
            this.outputFormat = 'html';
        }
        this.editor.chain().setContent(value, false).run();
    }
    // Registers a callback function that is called when the control's value changes in the UI.
    registerOnChange(fn) {
        this.onChange = fn;
    }
    // Registers a callback function that is called by the forms API on initialization to update the form model on blur.
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    // Called by the forms api to enable or disable the element
    setDisabledState(isDisabled) {
        this.editor.setEditable(!isDisabled);
        this._renderer.setProperty(this.el.nativeElement, 'disabled', isDisabled);
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        // take the inner contents and clear the block
        const innerHTML = this.el.nativeElement.innerHTML;
        this.el.nativeElement.innerHTML = '';
        // insert the editor in the dom
        this.el.nativeElement.append(...Array.from(this.editor.options.element.childNodes));
        // update the options for the editor
        this.editor.setOptions({ element: this.el.nativeElement });
        // update content to the editor
        if (innerHTML) {
            this.editor.chain().setContent(innerHTML, false).run();
        }
        // register blur handler to update `touched` property
        this.editor.on('blur', () => {
            this.onTouched();
        });
        // register transaction handler to emit changes on update
        this.editor.on('transaction', this.handleChange);
    }
    ngOnDestroy() {
        this.editor.destroy();
    }
}
EditorDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: EditorDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive });
EditorDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.1", type: EditorDirective, selector: "tiptap[editor], [tiptap][editor], tiptap-editor[editor], [tiptapEditor][editor]", inputs: { editor: "editor", outputFormat: "outputFormat" }, providers: [{
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => EditorDirective),
            multi: true
        }], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: EditorDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'tiptap[editor], [tiptap][editor], tiptap-editor[editor], [tiptapEditor][editor]',
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => EditorDirective),
                            multi: true
                        }]
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }]; }, propDecorators: { editor: [{
                type: Input
            }], outputFormat: [{
                type: Input
            }] } });

class FloatingMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        this.editor.registerPlugin(FloatingMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(FloatingMenuPluginKey);
    }
}
FloatingMenuDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: FloatingMenuDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
FloatingMenuDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.1", type: FloatingMenuDirective, selector: "tiptap-floating-menu[editor], [tiptapFloatingMenu][editor]", inputs: { editor: "editor", tippyOptions: "tippyOptions" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: FloatingMenuDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'tiptap-floating-menu[editor], [tiptapFloatingMenu][editor]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }]; }, propDecorators: { editor: [{
                type: Input
            }], tippyOptions: [{
                type: Input
            }] } });

class BubbleMenuDirective {
    constructor(_el) {
        this._el = _el;
        this.tippyOptions = {};
    }
    ngOnInit() {
        if (!this.editor) {
            throw new Error('Required: Input `editor`');
        }
        this.editor.registerPlugin(BubbleMenuPlugin({
            editor: this.editor,
            element: this._el.nativeElement,
            tippyOptions: this.tippyOptions
        }));
    }
    ngOnDestroy() {
        this.editor.unregisterPlugin(BubbleMenuPluginKey);
    }
}
BubbleMenuDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: BubbleMenuDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
BubbleMenuDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.1", type: BubbleMenuDirective, selector: "tiptap-bubble-menu[editor], [tiptapBubbleMenu][editor]", inputs: { editor: "editor", tippyOptions: "tippyOptions" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: BubbleMenuDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'tiptap-bubble-menu[editor], [tiptapBubbleMenu][editor]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }]; }, propDecorators: { editor: [{
                type: Input
            }], tippyOptions: [{
                type: Input
            }] } });

class DraggableDirective {
    constructor() {
        this.draggable = true;
        this.handle = '';
    }
}
DraggableDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: DraggableDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
DraggableDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.1", type: DraggableDirective, selector: "[tiptapDraggable]", host: { properties: { "attr.draggable": "this.draggable", "attr.data-drag-handle": "this.handle" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: DraggableDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tiptapDraggable]'
                }]
        }], propDecorators: { draggable: [{
                type: HostBinding,
                args: ['attr.draggable']
            }], handle: [{
                type: HostBinding,
                args: ['attr.data-drag-handle']
            }] } });

class NodeViewContentDirective {
    constructor() {
        this.handle = '';
        this.whiteSpace = 'pre-wrap';
    }
}
NodeViewContentDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NodeViewContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
NodeViewContentDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.1", type: NodeViewContentDirective, selector: "[tiptapNodeViewContent]", host: { properties: { "attr.data-node-view-content": "this.handle", "style.white-space": "this.whiteSpace" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NodeViewContentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tiptapNodeViewContent]'
                }]
        }], propDecorators: { handle: [{
                type: HostBinding,
                args: ['attr.data-node-view-content']
            }], whiteSpace: [{
                type: HostBinding,
                args: ['style.white-space']
            }] } });

class NgxTiptapModule {
}
NgxTiptapModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NgxTiptapModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxTiptapModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NgxTiptapModule, declarations: [EditorDirective,
        FloatingMenuDirective,
        BubbleMenuDirective,
        DraggableDirective,
        NodeViewContentDirective], exports: [EditorDirective,
        FloatingMenuDirective,
        BubbleMenuDirective,
        DraggableDirective,
        NodeViewContentDirective] });
NgxTiptapModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NgxTiptapModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: NgxTiptapModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        EditorDirective,
                        FloatingMenuDirective,
                        BubbleMenuDirective,
                        DraggableDirective,
                        NodeViewContentDirective,
                    ],
                    exports: [
                        EditorDirective,
                        FloatingMenuDirective,
                        BubbleMenuDirective,
                        DraggableDirective,
                        NodeViewContentDirective
                    ]
                }]
        }] });

class AngularRenderer {
    constructor(component, injector) {
        this.applicationRef = injector.get(ApplicationRef);
        const componentFactoryResolver = injector.get(ComponentFactoryResolver);
        const factory = componentFactoryResolver.resolveComponentFactory(component);
        this.componentRef = factory.create(injector, []);
        // Attach to the view so that the change detector knows to run
        this.applicationRef.attachView(this.componentRef.hostView);
    }
    get instance() {
        return this.componentRef.instance;
    }
    get elementRef() {
        return this.componentRef.injector.get(ElementRef);
    }
    get dom() {
        return this.elementRef.nativeElement;
    }
    detectChanges() {
        this.componentRef.changeDetectorRef.detectChanges();
    }
    destroy() {
        this.applicationRef.detachView(this.componentRef.hostView);
    }
}

class AngularNodeViewComponent {
}
AngularNodeViewComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: AngularNodeViewComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
AngularNodeViewComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.1.1", type: AngularNodeViewComponent, selector: "ng-component", inputs: { props: "props" }, ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.1", ngImport: i0, type: AngularNodeViewComponent, decorators: [{
            type: Component,
            args: [{ template: '' }]
        }], propDecorators: { props: [{
                type: Input
            }] } });
class AngularNodeView extends NodeView {
    mount() {
        const injector = this.options.injector;
        const props = {
            editor: this.editor,
            node: this.node,
            decorations: this.decorations,
            selected: false,
            extension: this.extension,
            getPos: () => this.getPos(),
            updateAttributes: (attributes = {}) => this.updateAttributes(attributes),
            deleteNode: () => this.deleteNode()
        };
        // create renderer
        this.renderer = new AngularRenderer(this.component, injector);
        // Pass input props to the component
        this.renderer.instance.props = props;
        if (this.extension.config.draggable) {
            // Register drag handler
            this.renderer.elementRef.nativeElement.ondragstart = (e) => {
                this.onDragStart(e);
            };
        }
        this.contentDOMElement = this.node.isLeaf ? null : document.createElement(this.node.isInline ? 'span' : 'div');
        if (this.contentDOMElement) {
            // For some reason the whiteSpace prop is not inherited properly in Chrome and Safari
            // With this fix it seems to work fine
            // See: https://github.com/ueberdosis/tiptap/issues/1197
            this.contentDOMElement.style.whiteSpace = 'inherit';
            this.renderer.detectChanges();
        }
        // attach stopEvent
        if (this.options.stopEvent) {
            this.stopEvent = this.options.stopEvent;
        }
    }
    updateProps(props) {
        this.renderer.instance.props = Object.assign(Object.assign({}, this.renderer.instance.props), props);
    }
    get dom() {
        return this.renderer.dom;
    }
    get contentDOM() {
        if (this.node.isLeaf) {
            return null;
        }
        this.maybeMoveContentDOM();
        return this.contentDOMElement;
    }
    maybeMoveContentDOM() {
        const contentElement = this.dom.querySelector('[data-node-view-content]');
        if (this.contentDOMElement
            && contentElement
            && !contentElement.contains(this.contentDOMElement)) {
            contentElement.appendChild(this.contentDOMElement);
        }
    }
    update(node, decorations) {
        if (this.options.update) {
            return this.options.update(node, decorations);
        }
        if (node.type !== this.node.type) {
            return false;
        }
        if (node === this.node && this.decorations === decorations) {
            return true;
        }
        this.node = node;
        this.decorations = decorations;
        this.updateProps({ node, decorations });
        this.maybeMoveContentDOM();
        return true;
    }
    selectNode() {
        this.updateProps({ selected: true });
    }
    deselectNode() {
        this.updateProps({ selected: false });
    }
    destroy() {
        this.renderer.destroy();
    }
}
const AngularNodeViewRenderer = (component, options) => {
    return (props) => {
        return new AngularNodeView(component, props, options);
    };
};

/*
 * Public API Surface of ngx-tiptap
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularNodeViewComponent, AngularNodeViewRenderer, AngularRenderer, BubbleMenuDirective, DraggableDirective, EditorDirective, FloatingMenuDirective, NgxTiptapModule, NodeViewContentDirective };
//# sourceMappingURL=ngx-tiptap.js.map
